-----------------------------------------------------------------------
-- Locale
--

local PREFIX = "cg: "
local CG_USAGE = "Whisper me '!cg <anything>' to search."
local NO_DATA = "No results found for %q."
local NOT_SCANNED = "%s tried searching your crafts, but you haven't opened the tradeskill windows yet."
local acceptedCommands = {
	cg = true,
	craft = true,
	enchant = true,
	gem = true,
}

-----------------------------------------------------------------------
-- Tooltip scanner, mostly stolen from Gratuity.
--

local scanItem
do
	local scanner = CreateFrame("GameTooltip")
	scanner:SetOwner(WorldFrame, "ANCHOR_NONE")
	local lcache, rcache = {}, {}
	for i = 1, 30 do
		lcache[i], rcache[i] = scanner:CreateFontString(), scanner:CreateFontString()
		lcache[i]:SetFontObject(GameFontNormal); rcache[i]:SetFontObject(GameFontNormal)
		scanner:AddFontStrings(lcache[i], rcache[i])
	end
	local function erase()
		for i = 1, scanner:NumLines() do rcache[i]:SetText() end
		scanner:ClearLines()
	end
	local scannerCache = {}
	function scanItem(itemLink)
		erase()
		scanner:SetHyperlink(itemLink)
		for k in pairs(scannerCache) do scannerCache[k] = nil end
		for i = 1, scanner:NumLines() do
			local left, right = lcache[i]:GetText(), rcache[i]:GetText()
			if left then table.insert(scannerCache, left) end
			if right then table.insert(scannerCache, right) end
		end
		return table.concat(scannerCache)
	end
end

-----------------------------------------------------------------------
-- CraftyGnome
--

local noData = true
local searchData = {}
local addon = CreateFrame("Frame")

function addon:CRAFT_SHOW()
	for i = 1, GetNumCrafts() do
		local skillName, _, skillType = GetCraftInfo(i)
		if skillType ~= "header" and skillName then
			local link = GetCraftRecipeLink(i)
			if not searchData[link] then
				searchData[link] = scanItem(link):lower()
			end
		end
	end
	noData = nil
end

function addon:TRADE_SKILL_SHOW()
	local name = GetTradeSkillLine()
	for i = 1, GetNumTradeSkills() do
		local skillName, skillType = GetTradeSkillInfo(i)
		if skillType ~= "header" and skillName then
			local link = GetTradeSkillRecipeLink(i)
			if not searchData[link] then
				searchData[link] = scanItem(link):lower()
			end
		end
	end
	noData = nil
end

do
	local ctl = ChatThrottleLib
	local reply = ctl and
		function(msg, player) ctl:SendChatMessage("BULK", "CraftyGnome", PREFIX .. msg, "WHISPER", nil, player) end or
		function(msg, player) SendChatMessage(PREFIX .. msg, "WHISPER", nil, player) end
	function addon:CHAT_MSG_WHISPER(msg, player)
		local cmd = select(3, msg:find("^!(%S+)"))
		if not cmd or not acceptedCommands[cmd] then return end
		local query = select(3, msg:find("^!%S+%s(.*)"))
		if query then
			query = query:lower()
			local nothing = true
			for item, details in pairs(searchData) do
				if details:find(query) then
					nothing = nil
					reply(v, player)
				end
			end
			if nothing or noData then
				reply(NO_DATA:format(query), player)
			end
		else
			reply(CG_USAGE, player)
		end
		if noData then
			DEFAULT_CHAT_FRAME:AddMessage(NOT_SCANNED:format(player))
		end
	end
end

local orig = ChatFrame_MessageEventHandler
local chatPrefixTest = "^" .. PREFIX
ChatFrame_MessageEventHandler = function(event, ...)
	if event == "CHAT_MSG_WHISPER_INFORM" and type(arg1) == "string" and arg1:find(chatPrefixTest) then
		return
	end
	return orig(event, ...)
end

addon:SetScript("OnEvent", function(self, event, ...)
	self[event](self, ...)
end)
addon:RegisterEvent("CHAT_MSG_WHISPER")
addon:RegisterEvent("CRAFT_SHOW")
addon:RegisterEvent("TRADE_SKILL_SHOW")

